'use strict';

//SCHEMAcat angular controllers
SCHEMAcat.controller('WorkspaceCtrl', ['$scope', '$location', '$routeParams', '$timeout', 'SchemasList', 'Schema', 'UserService', 'Settings',
    function($scope, $location, $routeParams, $timeout, SchemasList, Schema, UserService, Settings) {

        //these models have to be stored in the parent scope since they are used across page changes and
        //this controller is reloaded everytime the users changes page, thus reseting the $scope models.
        var parentScope = $scope.$parent;
        parentScope.pageSize = 10;

        //these models are stored in the local $scope since they are not used across page changes
        $scope.selectedSchemaIndex = $routeParams.schemaIndex;

        //keep the workspace page hidden until all data is loaded            
        $scope.animateWorkspacePageChange = 'invisible';



        //watch for our AJAX data to arrive and fade in workspace afterwards.
        //when changing page, the data is requested before the workspace controller is reloaded
        //so we should fade in the workspace in the new controller instance after reload.
        var watchListener = $scope.$watch('resultHasArrived', function(newValue, oldValue) {
            if (newValue) {
                if (Settings.getAnimationStatus()) {
                    //fade in the workspace page
                    switch (parentScope.toNextPage) {
                        case false:
                            $scope.animateWorkspacePageChange = 'animated slideInLeft';
                            break;
                        case true:
                            $scope.animateWorkspacePageChange = 'animated slideInRight';
                            break;
                            //if it is a reload toNextPage will be undefined
                        default:
                            $scope.animateWorkspacePageChange = 'animated fadeInDown';
                    }

                    $timeout(function() {
                        $scope.animateWorkspacePageChange = '';
                    }, 250);
                } else
                    $scope.animateWorkspacePageChange = '';
                //derregister the listener since the workspace page is now loaded
                watchListener();
            }
        });


        //load the schemas page and the selected schema metadata.
        //if reading the workspace for the first time i.e. not due to page change click
        if (!parentScope.workspaceFirstLoaded)
            loadSchemasList($routeParams.pageNumber);


        //update the current route when user clicks in another page in the paginator.
        //This triggers a $routeChangeStart event since we are changing the route path part.
        $scope.selectPage = function(page) {
            $location.search().schemaIndex = '0';
            delete $location.search().show;
            $location.path('workspace/page/' + page + '/').search($location.search());
        };


        //update the current route when the uses clicks one schema in the list. This triggers
        //a $routeUpdate event since we are just changing the route search part.
        $scope.selectSchema = function(rowIndex) {
            $location.search().schemaIndex = rowIndex.toString();
            delete $location.search().show;
            $location.search($location.search());
        };


        //listen for the $routeChangeStart event. Triggered when route path is changed (page change).
        $scope.$on('$routeChangeStart', function(event, next, current) {
            parentScope.resultHasArrived = false;
            if (Settings.getAnimationStatus()) {
                if (next.params !== undefined &&
                    next.params.pageNumber !== undefined &&
                    next.params.pageNumber > current.params.pageNumber)
                    parentScope.toNextPage = true;
                else
                    parentScope.toNextPage = false;

                //slide out workspace page
                switch (next.templateUrl) {
                    case 'partials/workspace.html':
                        $scope.animateWorkspacePageChange =
                            parentScope.toNextPage ?
                            'animated slideOutLeft' : 'animated slideOutRight';
                        break;
                    case 'partials/contact.html':
                    case 'partials/about.html':
                        $scope.animateWorkspacePageChange = 'animated slideOutLeft';
                        break;
                    default:
                        $scope.animateWorkspacePageChange = 'animated fadeOutDown';
                }
            }
            if (next.templateUrl === 'partials/workspace.html')
                loadSchemasList(next.params.pageNumber);
            else
                parentScope.workspaceFirstLoaded = false;
        });

        $scope.$on('$routeChangeSuccess', function(event, current, previous) {
            //when coming back to the workspace from other routes, the workspace should apear from the left to the right
            if (Settings.getAnimationStatus()) {
                if (previous === undefined || previous.templateUrl === undefined ||
                    previous.templateUrl === 'partials/accountDetails.html')
                    parentScope.toNextPage = undefined;
            }
        });

        //listen for the $routeUpdate event and update the selected schema metadata.
        //(if not done, the selection highlight is not updated e.g. when
        //navigating back and forth)
        $scope.$on('$routeUpdate', function(event, route) {
            if (route.templateUrl === 'partials/workspace.html' && route.params.schemaIndex !== $scope.selectedSchemaIndex) {
                parentScope.resultHasArrived = false;
                if (Settings.getAnimationStatus()) {
                    $scope.toNextSchema =
                        route.params.schemaIndex > $scope.selectedSchemaIndex ?
                        true : false;

                    $scope.animateSchemaChange =
                        $scope.toNextSchema ?
                        'animated fadeOutUp' : 'animated fadeOutDown';
                    $timeout(function() {
                        loadSchemaMetadata(route.params.schemaIndex);
                    }, 250);
                } else
                    loadSchemaMetadata(route.params.schemaIndex);
            }
        });


        function loadSchemasList(page) {
            //check if pageNumber is not a number (can happen e.g. due to manual input)
            //reload on page 1 schema index 0 if so
            if (isNaN(parseInt(page, 10))  || page < 1) {
                $location.path('workspace/page/' + 1 + '/').search({
                    'schemaIndex': 0
                });
                return;
            }

            //When the user changes page, the new AJAX data (next page) is requested by
            //the unloading controller (previous page) and used by the next loaded
            //controller (next page), thus we have to keep the server data across controller 
            //reloads by saving it in the parent scope.
            SchemasList.get({
                    userID: UserService.id,
                    startPage: page,
                    pageSize: $scope.pageSize
                },
                function(result) {
                    //we now have the pages range so we can configure the paginator and check if pageNumber is out of
                    //range (can happen e.g. due to manual input) reload on page 1 schema index 0 if so
                    if (page > Math.ceil(result['sc:result']['sc:total-schemas'] / $scope.pageSize)) {
                        $location.path('workspace/page/' + 1 + '/').search({
                            'schemaIndex': 0
                        });
                        return;
                    }

                    parentScope.totalSchemaItems = result['sc:result']['sc:total-schemas'];
                    parentScope.currentPage = Number(page);


                    //when there is just one element we create an array with it, so ng-repeat repeats just once
                    parentScope.schemasList = prepareArrayForNgRepeat(result['sc:result']['sc:schemas']['sc:schema']);

                    loadSchemaMetadata($location.search().schemaIndex);

                    parentScope.schemasResult = result;
                });
        }


        //loads (trough a 'Schema' AJAX call) the schema metadata of the schema placed
        //in the 'index' position of the schemas list.
        function loadSchemaMetadata(index) {
            //if just changing the selected schema hide the metadata pane until the metadata is loaded
            if ($scope.animateWorkspacePageChange !== 'invisible')
                $scope.animateSchemaChange = 'invisible';
            //check if schemaIndex is out of range (can happen e.g. due to manual input)
            //reload on schema index 0 if so
            if (isNaN(parseInt(index, 10)) || index < 0 || index > $scope.schemasList.length - 1) {
                $scope.selectedSchemaIndex = undefined;
                $location.search({
                    'schemaIndex': 0
                });
                return;
            }
            //update the selected row to the route search part.
            $scope.selectedSchemaIndex = index;

            //make AJAX call to get the schema metadata for the selected row
            var selectedSchemaRow = parentScope.schemasList[Number(index)];
            Schema.get({
                    schemaID: selectedSchemaRow['xml:id']
                },
                function(result) {
                    //when there is just one element we create an array with it, so ng-repeat repeats just once
                    parentScope.filesList = prepareArrayForNgRepeat(result['sc:schema']['sc:files']['sc:file']);

                    if (Settings.getAnimationStatus()) {
                        //fade in the schema metadata
                        switch ($scope.toNextSchema) {
                            case true:
                                $scope.animateSchemaChange = 'animated fadeInUp';
                                break;
                            case false:
                                $scope.animateSchemaChange = 'animated fadeInDown';
                                break;
                                //if it is a reload toNextPage will be undefined
                            case undefined:
                                $scope.animateSchemaChange = 'animated fadeInDown';
                        }
                    } else
                        $scope.animateSchemaChange = '';

                    parentScope.selectedSchema = result;
                    parentScope.resultHasArrived = true;
                });

        }

        function prepareArrayForNgRepeat(data) {
            var array;
            if (data instanceof Array)
                array = data;
            else
                array = new Array(data);
            return array;
        }

        parentScope.workspaceFirstLoaded = true;

    }
]);


SCHEMAcat.controller('FileController', ['$scope', '$http', '$timeout', '$location',
    function($scope, $http, $timeout, $location) {

        updateShowFiles();

        //show-hide file toggle handler
        $scope.showFile = function(fileName) {
            //update the search part of the route
            //triggers '$routeUpdate'
            $scope.showFileAjaxError = false;
            $timeout(function() { //(wrapping in $timeout is neded for Firefox)
                var locationSearch = $location.search();
                if ($scope.showSchemaBtn === true) {
                    if (locationSearch.show === undefined)
                        locationSearch.show = '#' + fileName;
                    else if (locationSearch.show.indexOf(fileName) === -1)
                        locationSearch.show = locationSearch.show + '#' + fileName;
                } else {
                    for (var prop in locationSearch) {
                        if (prop === 'show') {
                            var newLocationSearchShowArray = locationSearch[prop].split('#');
                            for (var i in newLocationSearchShowArray) {
                                if (newLocationSearchShowArray[i] === fileName)
                                    newLocationSearchShowArray.splice(i, 1);
                            }

                            var newLocationSearchShowString = newLocationSearchShowArray.join('#');
                            if (newLocationSearchShowString !== '') {
                                locationSearch[prop] = newLocationSearchShowString;
                            } else
                                delete locationSearch[prop];
                        }
                    }
                }
                $location.search(locationSearch);
            });
        };

        $scope.$on('$routeUpdate', function(event, route) {
            if ($scope.resultHasArrived)
                updateShowFiles();
        });

        function updateShowFiles() {
            if ($location.search().show !== undefined && $location.search().show.indexOf($scope.file['sc:name']) !== -1) {
                $scope.showSchemaBtn = true;
                displayFile($scope.selectedSchema['sc:schema']['xml:id'], $scope.file['sc:name']);
            } else {
                $scope.showSchemaBtn = false;
                $scope.file.open = false;
            }
        }

        function displayFile(schemaID, fileName) {
            if ($scope.schemaFileContent === undefined) {
                $http.get('/schemacat/schemas/' + schemaID + '/files/' + fileName).
                success(function(data, status, headers, config) {
                    $scope.prettify = 'prettyprint linenums';

                    //execute prettyPrint asynchronously. We use $timeout since prettyPrint() 
                    //must run after DOM rendering
                    $timeout(function() {
                        prettyPrint();
                        $scope.file.open = true;
                    });

                    $scope.schemaFileContent = data;
                }).
                error(function(data, status, headers, config) {
                    $scope.showSchemaBtn = false;
                    $scope.showFile(fileName);
                    $scope.showFileAjaxError = true;
                });
            } else
                $scope.file.open = true;
        }

    }
]);


SCHEMAcat.controller('TopMenuCtrl', ['$scope', '$modal', '$location', 'UserService', 'Settings', 'ShibService',
    function($scope, $modal, $location, UserService, Settings, ShibService) {

        $scope.displayName = UserService.displayName;
        $scope.shibURL = ShibService.url;

        //watch for changes in the displayName since it can be changed by the user in the account details page
        $scope.$watch(
            function() {
                return UserService.displayName;
            },
            function(newValue, oldValue) {
                if (newValue !== oldValue) {
                    $scope.displayName = newValue;
                }
            });

        updateTopMenu();

        $scope.isActive = function(viewLocation) {
            if (viewLocation === '/workspace' && $location.path().indexOf('/workspace/') !== -1)
                return true;
            return viewLocation === $location.path();
        };

        $scope.updateAnimationStatus = function() {
            $scope.animationsChecked = !$scope.animationsChecked;
            if ($scope.animationsChecked)
                delete $location.search().animationsDisabled;
            else
                $location.search().animationsDisabled = true;
            $location.search($location.search());
        };

        $scope.toggleTopMenu = function() {
            $scope.topMenuCollapsed = !$scope.topMenuCollapsed;
            if ($scope.topMenuCollapsed)
                delete $location.search().topMenuExpanded;
            else
                $location.search().topMenuExpanded = true;

            $location.search($location.search());
        };

        $scope.animationsUrlPart = function() {
            if ($location.search().animationsDisabled)
                return 'animationsDisabled';
            else
                return '';
        };

        $scope.topMenuExpandedUrlPart = function() {
            if ($location.search().topMenuExpanded)
                return 'topMenuExpanded';
            else
                return '';
        };

        $scope.$on('$routeUpdate', function(event, route) {
            updateTopMenu();
        });

        //Modal window handlers and options
        $scope.loginClick = function() {
            var modalInstance = $modal.open({
                scope: $scope,
                templateUrl: 'partials/loginModal.html',
                controller: 'LoginModalCtrl'
            });

            modalInstance.result.then(function(loginName) {
                alert('Now set the user cookie to [' + loginName + ']');
                document.cookie = 'USER=' + loginName + ';domain=;path=/';
                document.location.hash = '';
                document.location.reload(true);
            });
        };

        function updateTopMenu() {
            $scope.topMenuCollapsed = !$location.search().topMenuExpanded;

            Settings.setAnimationStatus(!$location.search().animationsDisabled);
            $scope.animationsChecked = Settings.getAnimationStatus();
        }

    }
]);

SCHEMAcat.controller('LoginModalCtrl', ['$scope', '$modalInstance',
    function($scope, $modalInstance) {
        $scope.loginClose = function() {
            $modalInstance.dismiss('cancel');
        };

        //set cookie code. REMOVE in production
        $scope.loginSubmit = function() {
            $modalInstance.close(this.loginNameInput.text);
        };
    }
]);

SCHEMAcat.controller('AboutPageCtrl', ['$scope', 'Settings',
    function($scope, Settings) {

        //When the user loads (or reloads) the 'About' page as the first page to be accessed, the 'toNextPage' variable
        //used by the workspace to determine the direction of its enter animation is: 'undefined'. Thus we set it so
        //that if the user moves to the workspace ('Home'), the workspace will enter with the right animation.
        //This is done independently of 'Settings.getAnimationStatus()' so when the animations are turned on, their
        //governing variables are already set.
        $scope.$parent.toNextPage = false;

        if (Settings.getAnimationStatus()) {
            $scope.animateInOut = 'animated slideInRight';
        }

        $scope.$on('$routeChangeStart', function(event, next, current) {
            if (Settings.getAnimationStatus()) {
                switch (next.templateUrl) {
                    case 'partials/workspace.html':
                        $scope.animateInOut = 'animated slideOutRight';
                        break;
                    case 'partials/contact.html':
                        $scope.animateInOut = 'animated slideOutLeft';
                        break;
                    default:
                        $scope.animateInOut = 'animated fadeOutDown';
                }
            }
        });

        $scope.$on('$routeChangeSuccess', function(event, current, previous) {
            if (Settings.getAnimationStatus()) {
                var previousUrl = previous !== undefined ?
                    previous.templateUrl : undefined;
                switch (previousUrl) {
                    case 'partials/workspace.html':
                        $scope.animateInOut = 'animated slideInRight';
                        break;
                    case 'partials/contact.html':
                        $scope.animateInOut = 'animated slideInLeft';
                        break;
                    default:
                        $scope.animateInOut = 'animated fadeInDown';
                }
            }
        });
    }
]);

SCHEMAcat.controller('ContactPageCtrl', ['$scope', 'Settings',
    function($scope, Settings) {

        //When the user loads (or reloads) the 'Contact' page as the first page to be accessed, the 'toNextPage' variable
        //used by the workspace to determine the direction of its enter animation is: 'undefined'. Thus we set it so
        //that if the user moves to the workspace ('Home'), the workspace will enter with the right animation.
        //This is done independently of 'Settings.getAnimationStatus()' so when the animations are turned on, their
        //governing variables are already set.
        $scope.$parent.toNextPage = false;

        if (Settings.getAnimationStatus()) {
            $scope.animateInOut = 'animated slideInRight';
        }

        $scope.$on('$routeChangeStart', function(event, next, current) {
            if (Settings.getAnimationStatus()) {
                switch (next.templateUrl) {
                    case 'partials/about.html':
                    case 'partials/contact.html':
                    case 'partials/workspace.html':
                        $scope.animateInOut = 'animated slideOutRight';
                        break;
                    default:
                        $scope.animateInOut = 'animated fadeOutDown';
                }
            }
        });

        $scope.$on('$routeChangeSuccess', function(event, current, previous) {
            //when coming back to the workspace from other routes, the workspace should apear from the left to the right
            if (Settings.getAnimationStatus()) {
                var previousUrl = previous !== undefined ?
                    previous.templateUrl : undefined;
                switch (previousUrl) {
                    case 'partials/about.html':
                    case 'partials/contact.html':
                    case 'partials/workspace.html':
                        $scope.animateInOut = 'animated slideInRight';
                        break;
                    default:
                        $scope.animateInOut = 'animated fadeInDown';
                }
            }
        });
    }
]);

SCHEMAcat.controller('AccountDetailsPageCtrl', ['$scope', '$location', 'User', 'UserService', 'Settings',
    function($scope, $location, User, UserService, Settings) {

        //this function is used to clean outdated emails from the route search part
        //i.e. after editing an email the user can still go back in the browser but
        //the old email addess does not exist anymore, thus we clean it when back is clicked.
        function cleanOutdatedSearchPathEmails() {
            var locationSearch = $location.search();
            for (var prop in locationSearch) {
                if (prop === 'edditingEmail') {
                    var newLocationSearchEdditingEmailArray = locationSearch[prop].split('#');
                    for (var i = 0; i < newLocationSearchEdditingEmailArray.length; i++) {
                        var found = false;
                        for (var j = 0; j < UserService.email.length; j++) {
                            if (newLocationSearchEdditingEmailArray[i] === UserService.email[j])
                                found = true;
                        }
                        if (!found)
                            newLocationSearchEdditingEmailArray.splice(i, 1);
                    }
                    var newLocationSearchEdditingEmailString = newLocationSearchEdditingEmailArray.join('#');
                    if (newLocationSearchEdditingEmailString !== '') {
                        locationSearch[prop] = newLocationSearchEdditingEmailString;
                    } else
                        delete locationSearch[prop];
                }
            }
            $location.search(locationSearch);
        }

        function updateEdditingDisplayName() {
            $scope.waitingForServerEditDisplayName = false;
            $scope.editDisplayNameAjaxError = false;
            if ($location.search().edditingDisplayName) {
                $scope.editDisplayNameText = $scope.displayName;
                $scope.edditingDisplayName = true;
            } else
                $scope.edditingDisplayName = false;
        }

        function updateAddingEmail() {
            $scope.waitingForServer = false;

            if ($location.search().addingEmail)
                $scope.addingEmail = true;
            else
                $scope.addingEmail = false;
        }

        $scope.principal = UserService.principal;
        $scope.displayName = UserService.displayName;
        $scope.email = UserService.email;

        $scope.addingEmail = false;

        $scope.addNewEmail = function() {
            $location.search().addingEmail = true;
            $location.search($location.search());
        };

        $scope.terminateAddEmail = function() {
            $scope.ajaxError = false;
            delete $location.search().addingEmail;
            $location.search($location.search());
        };

        $scope.addEmail = function(newEmail) {
            $scope.waitingForServer = true;
            var user = new User();
            user.emails = UserService.email.slice(0);
            user.emails.push(newEmail);

            user.$save({
                    userID: UserService.id
                },
                function(result) {
                    if (result['sc:response'] !== undefined && result['sc:response'].success) {
                        UserService.email.push(newEmail);
                        $scope.terminateAddEmail();
                    } else {
                        //server reports an handeled error
                        $scope.waitingForServer = false;
                        $scope.ajaxError = true;
                    }
                },
                //no response, or response error flagged in http response headers
                function() {
                    $scope.waitingForServer = false;
                    $scope.ajaxError = true;
                });
        };

        $scope.editDisplayName = function() {
            $location.search().edditingDisplayName = true;
            $location.search($location.search());
        };

        $scope.terminateEditDisplayName = function() {
            delete $location.search().edditingDisplayName;
            $location.search($location.search());
        };

        $scope.updateDisplayName = function(newDisplayName) {
            $scope.waitingForServerEditDisplayName = true;
            var user = new User();
            user.displayName = newDisplayName;

            user.$save({
                    userID: UserService.id
                },
                function(result) {
                    if (result['sc:response'] !== undefined && result['sc:response'].success) {
                        $scope.displayName = UserService.displayName = newDisplayName;
                        $scope.terminateEditDisplayName();
                    } else {
                        //server reports an handeled error
                        $scope.waitingForServerEditDisplayName = false;
                        $scope.editDisplayNameAjaxError = true;
                    }
                },
                //no response, or response error flagged in http response headers
                function() {
                    $scope.waitingForServerEditDisplayName = false;
                    $scope.editDisplayNameAjaxError = true;
                });
        };

        if (Settings.getAnimationStatus()) {
            $scope.animateAccountDetails = 'animated fadeInDown';
        }

        $scope.$on('$routeChangeStart', function(event, next, current) {
            if (Settings.getAnimationStatus()) {
                $scope.animateAccountDetails = 'animated fadeOutUp';
            }
        });

        $scope.$on('$routeChangeSuccess', function(event, current, previous) {
            if (Settings.getAnimationStatus()) {
                $scope.animateAccountDetails = 'animated fadeInDown';
            }
            updateAddingEmail();
            updateEdditingDisplayName();
            cleanOutdatedSearchPathEmails();
        });

        $scope.$on('$routeUpdate', function(event, route) {
            updateAddingEmail();
            updateEdditingDisplayName();
            cleanOutdatedSearchPathEmails();
        });
    }
]);

SCHEMAcat.controller('EmailInstanceCtrl', ['$scope', '$location', 'User', 'UserService',
    function($scope, $location, User, UserService) {

        function updateEdditingEmail() {
            $scope.waitingForServerEditEmail = false;
            if ($location.search().edditingEmail !== undefined && $location.search().edditingEmail.indexOf($scope.emailInstance) !== -1) {
                $scope.editEmailText = $scope.emailInstance;
                $scope.edditingEmail = true;
            } else
                $scope.edditingEmail = false;
        }

        updateEdditingEmail();

        $scope.editEmail = function(emailInstance) {
            var locationSearch = $location.search();
            $scope.removeEmailAjaxError = false;
            if (locationSearch.edditingEmail === undefined)
                locationSearch.edditingEmail = emailInstance;
            else
                locationSearch.edditingEmail = $location.search().edditingEmail + '#' + emailInstance;

            $location.search(locationSearch);
        };

        $scope.removeEmail = function(index) {
            $scope.waitingForServerEditEmail = true;

            var user = new User();
            user.emails = UserService.email.slice(0);
            user.emails.splice(index, 1);

            user.$save({
                    userID: UserService.id
                },
                function(result) {
                    if (result['sc:response'] !== undefined && result['sc:response'].success) {
                        UserService.email.splice(index, 1);
                    } else {
                        //server reports an handeled error
                        $scope.waitingForServerEditEmail = false;
                        $scope.removeEmailAjaxError = true;
                    }
                },
                //no response, or response error flagged in http response headers
                function() {
                    $scope.waitingForServerEditEmail = false;
                    $scope.removeEmailAjaxError = true;
                });
        };

        $scope.updateEmail = function(index) {
            $scope.waitingForServerEditEmail = true;

            var user = new User();
            user.emails = UserService.email.slice(0);
            user.emails[index] = $scope.editEmailText;

            user.$save({
                    userID: UserService.id
                },
                function(result) {
                    if (result['sc:response'] !== undefined && result['sc:response'].success) {
                        var newEmail = $scope.editEmailText;
                        $scope.cancelEditEmail(UserService.email[index]);
                        UserService.email[index] = newEmail;
                    } else {
                        //server reports an handeled error
                        $scope.waitingForServerEditEmail = false;
                        $scope.editEmailAjaxError = true;
                    }
                },
                //no response, or response error flagged in http response headers
                function() {
                    $scope.waitingForServerEditEmail = false;
                    $scope.editEmailAjaxError = true;
                });
        };


        $scope.cancelEditEmail = function(emailInstance) {
            var locationSearch = $location.search();
            $scope.editEmailAjaxError = false;

            for (var prop in locationSearch) {
                if (prop === 'edditingEmail') {
                    var newLocationSearchEdditingEmailArray = locationSearch[prop].split('#');
                    for (var i in newLocationSearchEdditingEmailArray) {
                        if (newLocationSearchEdditingEmailArray[i] === emailInstance)
                            newLocationSearchEdditingEmailArray.splice(i, 1);
                    }
                    var newLocationSearchEdditingEmailString = newLocationSearchEdditingEmailArray.join('#');
                    if (newLocationSearchEdditingEmailString !== '') {
                        locationSearch[prop] = newLocationSearchEdditingEmailString;
                    } else
                        delete locationSearch[prop];
                }
            }
            $location.search(locationSearch);
        };

        $scope.$on('$routeChangeSuccess', function(event, current, previous) {
            updateEdditingEmail();
        });

        $scope.$on('$routeUpdate', function(event, route) {
            updateEdditingEmail();
        });
    }
]);

//SCHEMAcat controlers for completeUserDetails.html page
SCHEMAcat.controller('CompleteUserDetailsCtrl', ['$scope', '$modal',
    function($scope, $modal) {

        $scope.ajaxError = false;
        $scope.waitingForServer = false;

        var userDetailsModalInstance = $modal.open({
            scope: $scope,
            backdrop: 'static',
            templateUrl: 'partials/fillDetailsModal.html',
            controller: 'CompleteUserDetailsModalCtrl'
        });
    }
]);

SCHEMAcat.controller('CompleteUserDetailsModalCtrl', ['$scope', '$modalInstance', 'User', 'UserService',
    function($scope, $modalInstance, User, UserService) {

        $scope.userDetailsSubmit = function() {
            $scope.waitingForServer = true;
            $scope.ajaxError = false;

            var user = new User();

            user.emails = [];
            user.emails[0] = this.emailInput.text;
            user.displayName = this.displayNameInput.text;

            user.$save({
                    userID: UserService.id
                },
                function(result) {
                    if (result['sc:response'] !== undefined && result['sc:response'].success)
                        document.location.pathname = 'schemacat/site/index.html';
                    else {
                        //server reports an handeled error
                        $scope.waitingForServer = false;
                        $scope.ajaxError = true;
                    }
                },
                //no response, or response error flagged in http response headers
                function() {
                    $scope.waitingForServer = false;
                    $scope.ajaxError = true;
                });
        };
    }
]);